<?php

namespace Chemical\Reaction\Http\Controllers;

use App\Http\Controllers\PublicController as BaseController;
use Chemical\Reaction\Interfaces\ReactionRepositoryInterface;

class ReactionPublicController extends BaseController
{
    // use ReactionWorkflow;

    /**
     * Constructor.
     *
     * @param type \Chemical\Reaction\Interfaces\ReactionRepositoryInterface $reaction
     *
     * @return type
     */
    public function __construct(ReactionRepositoryInterface $reaction)
    {
        $this->repository = $reaction;
        parent::__construct();
    }

    /**
     * Show reaction's list.
     *
     * @param string $slug
     *
     * @return response
     */
    protected function index()
    {
        $reactions = $this->repository
        ->pushCriteria(app('Litepie\Repository\Criteria\RequestCriteria'))
        ->scopeQuery(function($query){
            return $query->orderBy('id','DESC');
        })->paginate();


        return $this->response->setMetaTitle(trans('$reaction::reaction.names'))
            ->view('reaction::reaction.index')
            ->data(compact('reactions'))
            ->output();
    }


    /**
     * Show reaction.
     *
     * @param string $slug
     *
     * @return response
     */
    protected function show($slug)
    {
        $reaction = $this->repository->scopeQuery(function($query) use ($slug) {
            return $query->orderBy('id','DESC')
                         ->where('slug', $slug);
        })->first(['*']);

        return $this->response->setMetaTitle($$reaction->name . trans('reaction::reaction.name'))
            ->view('reaction::reaction.show')
            ->data(compact('reaction'))
            ->output();
    }

}
