<?php

namespace Chemical\Reaction\Policies;

use Litepie\User\Contracts\UserPolicy;
use Chemical\Reaction\Models\Reaction;

class ReactionPolicy
{

    /**
     * Determine if the given user can view the reaction.
     *
     * @param UserPolicy $user
     * @param Reaction $reaction
     *
     * @return bool
     */
    public function view(UserPolicy $user, Reaction $reaction)
    {
        if ($user->canDo('reaction.reaction.view') && $user->isAdmin()) {
            return true;
        }

        return $reaction->user_id == user_id() && $reaction->user_type == user_type();
    }

    /**
     * Determine if the given user can create a reaction.
     *
     * @param UserPolicy $user
     * @param Reaction $reaction
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('reaction.reaction.create');
    }

    /**
     * Determine if the given user can update the given reaction.
     *
     * @param UserPolicy $user
     * @param Reaction $reaction
     *
     * @return bool
     */
    public function update(UserPolicy $user, Reaction $reaction)
    {
        if ($user->canDo('reaction.reaction.edit') && $user->isAdmin()) {
            return true;
        }

        return $reaction->user_id == user_id() && $reaction->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given reaction.
     *
     * @param UserPolicy $user
     * @param Reaction $reaction
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Reaction $reaction)
    {
        return $reaction->user_id == user_id() && $reaction->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given reaction.
     *
     * @param UserPolicy $user
     * @param Reaction $reaction
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Reaction $reaction)
    {
        if ($user->canDo('reaction.reaction.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given reaction.
     *
     * @param UserPolicy $user
     * @param Reaction $reaction
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Reaction $reaction)
    {
        if ($user->canDo('reaction.reaction.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
