<?php

namespace Chemical\Reaction\Providers;

use Illuminate\Support\ServiceProvider;

class ReactionServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'reaction');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'reaction');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerReaction();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerReaction()
    {
        $this->app->bind('reaction', function($app) {
            return new Reaction($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Reaction', 'Lavalite\Reaction\Facades\Reaction');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('chemical.reaction', function ($app) {
            return $this->app->make('Chemical\Reaction\Reaction');
        });

                // Bind Reaction to repository
        $this->app->bind(
            'Chemical\Reaction\Interfaces\ReactionRepositoryInterface',
            \Chemical\Reaction\Repositories\Eloquent\ReactionRepository::class
        );

        $this->app->register(\Chemical\Reaction\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Chemical\Reaction\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and reaction's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/reaction.php', 'reaction'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeReaction::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['chemical.reaction'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('chemical/reaction.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/reaction')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/reaction')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
